unit jlDBFig2Word;
  
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, jlFig2Word, DBCtrls, DB, DBTables;

type
  EInvalidFieldType = class(Exception);

type
  TjlDBFig2Word = class(TjlFig2Word)
  private
    FDataLink : TFieldDataLink;
    function GetDataField : string;
    procedure CheckFieldType( const Value : string);
    procedure SetDataField(const Value : string);
    function GetDataSource : TDataSource;
    procedure SetDataSource(Value : TDataSource);
    procedure DataChange( Sender : TObject);
    procedure CMGetDataLink( var Msg : TMessage); message cm_GetDataLink;
  protected
    procedure Notification( AComponent : TComponent;
                            Operation : TOperation); override;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;

  published

    property DataField : string
      read GetDataField
      write SetDataField;

    property DataSource : TDataSource
      read GetDataSource
      write SetDataSource;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Joseph', [TjlDBFig2Word]);
end;

constructor TjlDBFig2Word.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  inherited AcceptZero := True;
  FDataLink := TFieldDataLink.Create;
  FDataLink.OnDataChange := DataChange;
end;

destructor TjlDBFig2Word.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

procedure TjlDBFig2Word.Notification( AComponent : TComponent;
                                      Operation : TOperation);
begin
  inherited Notification(AComponent,Operation);
  if (Operation = opRemove) and
     (FDataLink <> nil) and
     (AComponent = DataSource) then
    DataSource := nil;
end;

function TjlDBFig2Word.GetDataField : string;
begin
  Result := FDataLink.Fieldname;
end;

procedure TjlDBFig2Word.SetDataField( const Value : string);
begin
  CheckFieldType(Value);
  FDataLink.FieldName := Value;
end;

function TjlDBFig2Word.GetDataSource : TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TjlDBFig2Word.SetDataSource( Value : TDataSource);
begin
  if FDataLink.DataSource <> Value then
   begin
    FDataLink.DataSource := Value;
    if Value <> nil then Value.FreeNotification (Self);
   end;
end;

procedure TjlDBFig2Word.DataChange (Sender : TObject);
begin
  if FDataLink.Field = nil then
    Caption := ''
  else
    Figure := FDataLink.Field.AsFloat;
end;

procedure TjlDBFig2Word.CMGetDataLink( var Msg : TMessage);
begin
  Msg.Result := Integer( FDataLink );
end;

procedure TjlDBFig2Word.CheckFieldType( const Value : string);
var
  FieldType : TFieldType;

begin
  if (Value <> '') and (FDataLink <> nil) and
     (FDataLink.DataSet <> nil) and (FDataLink.DataSet.Active ) then
   begin
    Fieldtype := FDataLink.DataSet.FieldByName(Value).DataType;
    if (FieldType <> ftInteger) and (FieldType <> ftSmallInt) and
       (FieldType <> ftWord) and (FieldType <> ftFloat) and
       (FieldType <> ftCurrency) and (FieldType <> ftAutoInc) then
     begin
      raise EInvalidFieldType.Create('jlDBFig2Word.DataField can only ' +
                                     'be a numeric field.');
     end;
   end; { if (Value <> '') and ... }
end;

end.
