unit jlFig2Word;
   
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  EFigureOutofRangeError = class( Exception);
  TOutofRangeHandleType = (fwRaiseException, fwShowFigures);

type
  TjlFig2Word = class(TLabel)
  private
    FFigure : double;
    FAcceptZero : boolean;
    FWhenOutOfRange : TOutofRangeHandleType;
    function ConvertFigureToWords(const xxFigure : double) : string;
    function ValidateFigure(const xxFigure : double) : boolean;
    procedure SetFigure( const Value : double);
  protected
  public
    constructor Create(AOwner : TComponent); override;
  published

    property Figure : double
      Read  FFigure
      Write SetFigure;

    property AcceptZero : boolean
      read FAcceptZero
      write FAcceptZero
      default False;

    property WhenOutOfRange : TOutofRangeHandleType
      read FWhenOutOfRange
      write FWhenOutOfRange
      default fwShowFigures;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Joseph', [TjlFig2Word]);
end;

constructor TjlFig2Word.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FAcceptZero := False;
  FWhenOutOfRange := fwShowFigures;
end;

procedure TjlFig2Word.SetFigure(const Value : double);
begin
  if FFigure <> Value then
   begin
    FFigure := Value;
    inherited Caption := ConvertFigureToWords ( Value);
    Invalidate;
   end;
end;

function TjlFig2Word.ValidateFigure(const xxFigure : double) : boolean;

begin
  Result := (xxFigure <= 999999999999.99) and
     ( (FAcceptZero and (xxFigure >= 0.0))
     or ( (not FAcceptZero) and (xxFigure > 0.0) ) );
end;


function TjlFig2Word.ConvertFigureToWords(const xxFigure : double) : string;

var
  s1, s2, Final : string;
  i,j,k : integer;
  Big : array[0..4] of string;

const

  Tens: array[2..9] of string = ('Twenty ', 'Thirty ', 'Forty ', 'Fifty ',
                                 'Sixty ', 'Seventy ', 'Eighty ', 'Ninety ');

  Odds: array[0..19] of string = ('', 'One ', 'Two ', 'Three ', 'Four ',
                                  'Five ', 'Six ', 'Seven ', 'Eight ', 'Nine ',
                                  'Ten ', 'Eleven ', 'Twelve ', 'Thirteen ',
                                  'Fourteen ', 'Fifteen ', 'Sixteen ',
                                  'Seventeen ', 'Eighteen ', 'Nineteen ');

begin

  Result := '';

  if not ValidateFigure(xxFigure) then
   begin
    case FWhenOutOfRange of
     fwRaiseException :
        raise EFigureOutOfRangeError.Create('Figure out of support range. ' +
                                  chr(13) + 'Maximum is 999,999,999,999.99, ' +
                                  chr(13) + 'minimum is zero if AcceptZero is set to true ' +
                                  chr(13) + 'or greater than zero if AcceptZero is false.');
     fwShowFigures :
       begin
        Result := FormatFloat('#,##0.00', xxFigure);
        Exit;
       end;
    end;  { case }
   end;   { not ValidateFigure(xxFigure) }

  if (FAcceptZero) and (xxFigure = 0.0) then
   begin
    Result := '0.00';
    Exit;
   end;

  Big[0] := 'Billion ';
  Big[1] := 'Million ';
  Big[2] := 'Thousand ';
  Big[3] := '';
  Big[4] := '';

  k := 0;
  s2 := '';  Final := '';
  s1 := Formatfloat('000000000000.00', xxFigure);

  { The integer part contains 12 digits where they will be processed
    in groups of 3 digits which makes a total of 4 groups whereas
    the leftmost will be the first. }

  if xxFigure < 1.0 then
   begin
    Big[3] := 'Cents ';
    j := 4;   { Start integer part processing from group 4 }
   end
  else { xxFigure < 1.0 }
   begin
    if frac( xxFigure ) > 0.0 then
     begin
      Big[3] := 'and Cents ';
      j := 4;
     end
    else
     begin
      j := 3;
     end
   end;   { xxFigure < 1.0 }

  for i := 0 to j do
   begin
    s2 := copy(s1, i*3+1, 3);
    if s2[1] = '.' then s2[1] := '0';
    if StrToInt(s2)  = 0 then
      Continue
    else
     begin
      try
        k := StrtoInt(copy(s2,1,1));
      except
        on EConvertError do k := 0;
      end;
      if k > 0 then Final := Final + Odds[k] + 'Hundred ';
      k := StrtoInt(copy(s2,2,2));
      if k <> 0 then
       if k < 20 then
        Final := Final + Odds[k]
       else
        Final := Final + Tens[k div 10] + Odds[k mod 10];
      Final := Final + Big[i];
     end;
   end;  {for}

  Result := Final + 'Only';

end;

end.
